#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <dirent.h>
#include <dlfcn.h>

#include "my_app.h"
#include "plugins.h"

Plugin *plugin_load (const char *filename)
{
  Plugin *p;
  int (*init)(Plugin *);
  
  p = malloc (sizeof(Plugin));
  if (p == NULL)
  {
    printf ("Couldn't allocate Plugin object\n");
    return NULL;
  }
  strncpy (p->filename, filename, sizeof(p->filename));
  if ((p->module = dlopen(p->filename, RTLD_LAZY)) == NULL)
  {
    printf ("%s\n", dlerror());
    free (p);
    return NULL;
  }

  init = dlsym (p->module, "init_plugin");
  if (init == NULL)
  {
    printf ("%s\n", dlerror());
    free (p);
    return NULL;
  }

  if (!init(p))
  {
    perror ("Couldn't initialize plugin");
    free (p);
    return NULL;
  }

  p->load = TRUE;
  return p;
}

void plugin_action (Plugin *p)
{
  if (p == NULL)
  {
    printf ("Plugin not found!\n");
    return;
  }
  p->plugin_cb();
}

char *plugin_select ()
{
  DIR *dp;
  struct dirent *e;
  static char buf[BUFLEN];
  char *p;
  size_t len;
  char *suffix;

  dp = opendir (".");
  if (dp == NULL)
  {
    perror ("Cannot open '.'");
    return NULL;
  }
  printf ("\nFollowing plugins are available:\n");
  while ((e=readdir(dp)) != NULL)
  {
    suffix = strrchr (e->d_name, '.');
    if (suffix)
    {
      if (strcmp (suffix, PLUGIN_SUFFIX) == 0)
        printf ("  %s\n", e->d_name);
    }
  }
  closedir (dp);
  printf ("Your choice: ");
  buf[0] = '.';
  buf[1] = '/';
  p = &buf[2];
  fgets (p, sizeof(buf)-2, stdin);
  len = strlen (buf);
  if (len > 0)
    len --;
  buf[len] = '\0';
  if (strchr(p, '.') == NULL)
    strcat (buf, PLUGIN_SUFFIX);

  return buf;
}

void plugin_destroy (Plugin *p)
{
  void (*unload)(Plugin *);

  unload = dlsym (p->module, "unload_plugin");
  if (unload != NULL)
    unload (p);
  
  dlclose (p->module);
  free (p);
}
